{{ radar.spin - control both PING))) sensor and servo

   14 Aug 2007 - Xander Soldaat
                 Initial version

}}

DAT
  servoAng    Word 500, 616, 732, 848, 964, 1080, 1196, 1312, 1428, 1544, 1660, 1776, 1892, 2008, 2124, 2240

CON
  LEFT          = 1
  RIGHT         = 0
  CACHESIZE     = 16
   
VAR
  byte  indexAng                'current angle of servo
  byte  servoPin                'servo output pin (4.7K)
  byte  sensorPin               'PING))) output pin
  word  leftMax                 'max left servo position
  word  rightMax                'max right servo position
  word  ticksPeruSec            'number of ticks per uSec
  long  ticksPermSec            'number of ticks per mSec
  word  buffer[CACHESIZE]       'buffer with distances to objects
  word  cache[CACHESIZE]        'cached copy of the buffer
  byte  SemID                   'semaphore for buffer/cache writes
  long  stack[40]               'stack for the cog
  byte  _map_ready              'boolean for querying whether map is ready
  byte  cog_nr                  'keeps track of the cog index, so we can stop it


OBJ
  Debug: "SerialMirror"
  'Ping: "ping"

PUB start(_sensorPin, _servoPin, _rightMax, _leftMax)

  _map_ready := false
  ticksPeruSec := clkfreq / 1_000_000
  ticksPermSec := clkfreq / 1_000
  wordfill(@cache, CACHESIZE, 0)
  wordfill(@buffer, CACHESIZE, 0)

  SemID := locknew
  if SemID < 0
    Debug.str(string("Serious problem: locknew did not return a valid ID", 10, 13))
    waitcnt(clkfreq * 10 + cnt)
    return false
  lockclr(semID)
  return true
    
PUB reset_servo | w
  Debug.str(string("Resetting servo to initial position", 10, 13))
  repeat 20
    w := cnt
    sendPulse(servoAng[0])
    waitcnt(w + 100 * ticksPermSec)
  indexAng := 0

PUB swivel
  cog_nr := cognew(_swivel, @stack)

PUB stop
  Debug.str(string("Stopping cog number "))
  Debug.dec(cog_nr)
  Debug.str(string(10, 13))
  cogstop(cog_nr~)
  
PUB map_ready
  return _map_ready
  
PRI _swivel | sweepDir, w
  ''' Swivel the servo from right to left and back
  Debug.str(string("swivel"))
  Debug.str(string(10, 13))
  repeat
    repeat indexAng from 0 to CACHESIZE
      w := cnt
      sendPulse(servoAng[indexAng])
      repeat until not lockset(SemID)
      buffer[indexAng] := getDist
      lockclr(semID)
      waitcnt(w + 100 * ticksPermSec)
    flushBuffer
      
    repeat indexAng from CACHESIZE to 0
      w := cnt
      sendPulse(servoAng[indexAng])
      repeat until not lockset(SemID)
      buffer[indexAng] := getDist
      lockclr(semID)
      waitcnt(w + 100 * ticksPermSec)
    flushBuffer       

PRI sendPulse(_servoAng) | _pulseLength
  _pulseLength := _servoAng * ticksPeruSec   
  'outa[servoPin]~~       'set pin
  'waitcnt(s+w)           'wait pulse width
  'outa[servoPin]~        'clear pin      
   
PRI getDist : dist
  'ping.Centimeters(sensorPin)
  ' Generate a somewhat random number for testing purposes
  dist := cnt
  repeat 10
    dist?
  dist := dist.byte[1]

PRI flushBuffer | i
  repeat until not lockset(SemID)
  Debug.str(string("Flushing buffer to cache", 10, 13))
  wordfill(@cache, CACHESIZE, 0)
  wordmove(@cache, @buffer, CACHESIZE)
  _map_ready := true
  lockclr(semID)
  
PUB copyCache(mapPtr) | i
  repeat until not lockset(SemID)
  Debug.str(string("Copying cache to map", 10, 13))
  Debug.str(string("cache: "))
  repeat i from 0 to CACHESIZE - 1
    Debug.dec(buffer[i])
    Debug.str(string(" "))
  Debug.str(string(10,13))
  wordfill(mapPtr, CACHESIZE, 0)   
  wordmove(mapPtr, @cache, CACHESIZE)
  lockclr(semID)   